/***
 * Juliac
 * Copyright (C) 2010-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliac.spoon;

import java.lang.annotation.Annotation;
import java.util.Set;

import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedField;

import spoon.reflect.declaration.CtField;
import spoon.reflect.declaration.ModifierKind;
import spoon.reflect.reference.CtTypeReference;

/**
 * Implementation of {@link UnifiedField} for {@link CtField}s.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 2.2.5
 */
public class UCtField implements UnifiedField {
    
	private Juliac jc;
	
    /** The concrete {@link CtField}. */
    private CtField<?> ctfield;
    
    /** The class declaring the current field. */
    private UCtClass uctclass;
    
    public UCtField( Juliac jc, UCtClass uctclass, CtField<?> ctfield ) {
        
    	this.jc = jc;
    	this.ctfield = ctfield;
        this.uctclass = uctclass;
    }
    
    public UCtClass getDeclaringClass(){
    	return uctclass;
    }
    
    public <A extends Annotation> A getAnnotation( Class<A> annotationType ) {
        A a = ctfield.getAnnotation(annotationType);
        return a;
    }

    public String getGenericType() {
        CtTypeReference<?> type = ctfield.getType();
        String typename = SpoonHelper.getClassName(type);
        return typename;
    }

    public int getModifiers() {
        int result = 0;
        Set<ModifierKind> modks = ctfield.getModifiers();
        for (ModifierKind modk : modks) {
            int mod = SpoonHelper.toModifier(modk);
            result += mod;
        }
        return result;
    }
    
    public String getName() {
        return ctfield.getSimpleName();
    }
    
    public UnifiedClass getType() {
        CtTypeReference<?> tref = ctfield.getType();
        UnifiedClass uc = jc.create(tref.getQualifiedName());
        return uc;
    }        

    @Override
    public String toString() {
    	return ctfield.toString();
    }
}
